<?php

defined('ABSPATH') || exit;

// Include Tables
if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

class send_push_notifications_list_recipients extends WP_List_Table
{

    public static $table;
    public $root_path = FUSION_WA_URL . '-recipients';
    public static $filter = 'registered';

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->set_table();
        $this->set_filter();
        parent::__construct([
            'singular' => 'Recipient', //singular name of the listed records
            'plural'   => 'Recipients', //plural name of the listed records
            'ajax'     => false, //should this table support ajax?
        ]);
    }

    /**
     * Submenu
     */
    function get_views()
    {
        $views = array();
        // $current = (!empty($_REQUEST['filtered']) ? $_REQUEST['filtered'] : 'registered');

        // Registered link
        // $class = ($current == 'registered' ? ' class="current"' : '');
        // $registered = remove_query_arg('filtered');
        // $views['registered'] = "<a href='{$registered}' {$class} >" . __('Regitered', FUSION_WA_SLUG) . " (" . self::record_count() . ")</a>";
        $views['registered'] = "<a href='{$registered}' class='current'>" . __('Regitered', FUSION_WA_SLUG) . " (" . self::record_count() . ")</a>";
        
        // Unregistered link
        // $unregistered = add_query_arg('filtered', 'unregistered');
        // $class = ($current == 'unregistered' ? ' class="current"' : '');
        // $views['unregistered'] = "<a href='{$unregistered}' {$class} >" . __('Unregistered', FUSION_WA_SLUG) . " (" . self::record_count('unregistered') . ")</a>";

        return $views;
    }

    /**
     * Set filter action
     */
    private function set_filter()
    {
        return self::$filter = (!empty($_REQUEST['filtered']) ? $_REQUEST['filtered'] : 'registered');
    }
    
    /**
     * Set table name
     */
    private function set_table()
    {
        global $wpdb;
        self::$table = $wpdb->prefix . FUSION_WA_T_RECIPIENT_DEVICES;
    }

    /**
     * Get items
     */
    public static function get_items($per_page = 20, $page_number = 1)
    {
        global $wpdb;
        $sql = "SELECT * FROM " . self::$table;

        if (self::$filter == 'registered') {
            $sql .= " WHERE NOT user_id = 0 ";
        } else {
            $sql .= " WHERE user_id = 0 ";
        }
        if (!empty($_REQUEST['orderby'])) {
            $sql .= ' ORDER BY ' . esc_sql($_REQUEST['orderby']);
            $sql .= !empty($_REQUEST['order']) ? ' ' . esc_sql($_REQUEST['order']) : ' ASC';
        } else {
            $sql .= ' ORDER BY id desc';
        }
        $sql .= " LIMIT $per_page";
        $sql .= ' OFFSET ' . ($page_number - 1) * $per_page;
        $devices = $wpdb->get_results($sql, 'ARRAY_A');

        if (self::$filter == 'registered') {
            foreach ($devices as $key => $device) {
                $user = get_user_by('ID', $device['user_id']);
                error_log(json_encode($user));
                $devices[$key]['user_name'] = $user->display_name;
                $devices[$key]['user_email'] = $user->user_email;
            }
        }

        return $devices;
    }

    /**
     * Get item
     */
    public static function get_item($id)
    {
        global $wpdb;
        $sql = "SELECT * FROM " . self::$table . " where id = $id LIMIT 1";
        return $wpdb->get_row($sql);
    }

    /**
     * Delete record
     *
     * @param int $id record ID
     */
    public static function delete_record($id)
    {
        global $wpdb;
        $row = self::get_item($id);
        if ($row->status === 'trashed') {
            $wpdb->delete(
                self::$table,
                ['id' => $id],
                ['%d']
            );
        } else {
            $wpdb->update(
                self::$table,
                ['status' => 'trashed'],
                ['id' => $id]
            );
        }
    }

    /**
     * Restore record
     *
     * @param int $id record ID
     */
    public static function restore_record($id)
    {
        // global $wpdb;
        // $wpdb->update(
        //     self::$table,
        //     ['status' => 'active'],
        //     ['id' => $id]
        // );
    }

    /**
     * Returns the count of records in the database.
     *
     * @return null|string
     */
    public static function record_count($status = 'registered')
    {
        if ($status == 'registered') {
            global $wpdb;
            $sql = "SELECT COUNT(*) FROM " . self::$table . " WHERE NOT user_id = 0";
            return $wpdb->get_var($sql);
        } else {
            global $wpdb;
            $sql = "SELECT COUNT(*) FROM " . self::$table . " WHERE user_id = 0";
            return $wpdb->get_var($sql);
        }
    }

    /** Text displayed when no items available */
    public function no_items()
    {
        echo esc_html__('No records to display.', FUSION_WA_SLUG);
    }

    /**
     * Method for name column
     *
     * @param array $item an array of DB data
     *
     * @return string
     */
    // public function column_user_name($item)
    // {
        // create a nonce
        // $actions = [];
        // $title = '<strong>' . $item['user_name'] . '</strong>';
        // $delete_nonce = wp_create_nonce('recipient_action_single_record');

        // $actions['send_push'] = sprintf(
        //     '<a href="?page=%s&action=%s&id=%s&_wpnonce=%s">Send push</a>',
        //     esc_attr($_REQUEST['page']),
        //     'send-push',
        //     absint($item['id']),
        //     $delete_nonce
        // );

        // return $title . $this->row_actions($actions);
    // }

    /**
     * Render a column when no column specific method exists.
     *
     * @param array $item
     * @param string $column_name
     *
     * @return mixed
     */
    public function column_default($item, $column_name)
    {
        return $item[$column_name];
    }

    /**
     *  Associative array of columns
     *
     * @return array
     */
    function get_columns()
    {
        // $current = (!empty($_REQUEST['filtered']) ? $_REQUEST['filtered'] : 'registered');
        // if ($current == 'registered') {
        //     return [
        //         // 'user_name' => __('Username', FUSION_WA_SLUG),
        //         'user_email' => __('Email', FUSION_WA_SLUG),
        //         'name_device' => __('Name device', FUSION_WA_SLUG),
        //         'push_token' => __('Token', FUSION_WA_SLUG),
        //         'platform' => __('Platform', FUSION_WA_SLUG),
        //         'installation_id' => __('Installation ID', FUSION_WA_SLUG),
        //         'registered' => __('Registered', FUSION_WA_SLUG)
        //     ];
        // }
        return [
            'name_device' => __('Name device', FUSION_WA_SLUG),
            'push_token' => __('Token', FUSION_WA_SLUG),
            'platform' => __('Platform', FUSION_WA_SLUG),
            'installation_id' => __('Installation ID', FUSION_WA_SLUG),
            'registered' => __('Registered', FUSION_WA_SLUG)
        ];
    }

    /**
     * Columns to make sortable.
     *
     * @return array
     */
    public function get_sortable_columns()
    {
        $sortable_columns = array(
            'registered' => array('registered', false)

        );
        return $sortable_columns;
    }

    /**
     * Define which columns are hidden
     *
     * @return Array
     */
    public function get_hidden_columns()
    {
        return array();
    }

    /**
     * Handles data query and filter, sorting, and pagination.
     */
    public function prepare_items()
    {
        $columns = $this->get_columns();
        $hidden = $this->get_hidden_columns();
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = array($columns, $hidden, $sortable);

        $this->process_bulk_action();
        $this->process_action();

        $per_page     = $this->get_items_per_page('items_per_page', 20);
        $current_page = $this->get_pagenum();
        $total_items  = self::record_count();

        $this->set_pagination_args([
            'total_items' => $total_items,
            'per_page'    => $per_page
        ]);
        $this->items = self::get_items(20, $current_page);
    }

    /**
     * Process other actions
     */
    public function process_action()
    {
        // Detect when single delete action is being triggered...
        if ($this->current_action() === 'delete-single') {
            // security check!
            $delete_id = esc_sql($_REQUEST['id']);
            $nonce  = filter_input(INPUT_GET, '_wpnonce', FILTER_SANITIZE_STRING);
            if (!wp_verify_nonce($nonce, 'recipient_action_single_record')) {
                die('Go get a life script kiddies');
            }
            // loop over the array of record IDs and delete them
            self::delete_record($delete_id);
            $_SESSION[FUSION_WA_SLUG . '-message-type'] = 'success';
            $_SESSION[FUSION_WA_SLUG . '-message'] = __('Item successfully removed!', FUSION_WA_SLUG);
            wp_redirect($this->root_path);
            exit;
        }

        // Detect when single restore action is being triggered...
        if ($this->current_action() === 'restore-single') {
            // security check!
            $restore_id = esc_sql($_REQUEST['id']);
            $nonce  = filter_input(INPUT_GET, '_wpnonce', FILTER_SANITIZE_STRING);
            if (!wp_verify_nonce($nonce, 'recipient_action_single_record')) {
                die('Go get a life script kiddies');
            }
            // loop over the array of record IDs and delete them
            self::restore_record($restore_id);
            $_SESSION[FUSION_WA_SLUG . '-message-type'] = 'success';
            $_SESSION[FUSION_WA_SLUG . '-message'] = __('Item successfully restored!', FUSION_WA_SLUG);
            wp_redirect($this->root_path);
            exit;
        }
    }
}
